import { Injector, ReflectiveInjector, Directive, ElementRef, ComponentFactoryResolver, Input, ViewContainerRef, NgZone } from '@angular/core';
import * as ons from 'onsenui';
import { Params } from '../ons/params';
export class NavigatorPage {
    constructor(elementRef = null, destroy = function () { }, animator = null, params = new Params()) {
        this.elementRef = elementRef;
        this.destroy = destroy;
        this.animator = animator;
        this.params = params;
    }
}
/**
 * @element ons-navigator
 * @directive OnsNavigator
 * @selector ons-navigator
 * @codpen KgvAvx
 * @description
 *   [en]Angular 2 directive for `<ons-navigator>` component.[/en]
 *   [ja]`<ons-navigator>`要素のAngular 2ディレクティブです。[/ja]
 * @example
 *   @Component({
 *     selector: 'ons-page',
 *     template: `
 *       <ons-toolbar>
 *         <div class="center">Page</div>
 *       </ons-toolbar>
 *       <div class="content">...</div>
 *     `
 *   })
 *   class DefaultPageComponent { }
 *
 *   @Component({
 *     selector: 'navigator-app',
 *     template: `
 *     <ons-navigator [page]="page"></ons-navigator>
 *     `
 *   })
 *   export class AppComponent {
 *     page = DefaultPageComponent
 *   }
 */
export class OnsNavigator {
    constructor(_elementRef, _resolver, _viewContainer, _injector, _zone) {
        this._elementRef = _elementRef;
        this._resolver = _resolver;
        this._viewContainer = _viewContainer;
        this._injector = _injector;
        this._zone = _zone;
        this._lastPageLoader = this.element.pageLoader;
        this.element.pageLoader = this._createPageLoader();
    }
    /**
     * @input page
     * @type {Type<any>}
     * @desc
     *   [en]Type of the page component.[/en]
     *   [ja]ページコンポーネントのクラスを指定します。[/ja]
     */
    set pageComponentType(page) {
        this.element.page = page;
    }
    get element() {
        return this._elementRef.nativeElement;
    }
    get nativeElement() {
        return this._elementRef.nativeElement;
    }
    _createPageLoader() {
        const componentRefMap = new WeakMap();
        return new ons.PageLoader(({ page, parent, params }, done) => {
            this._zone.run(() => {
                const pageParams = new Params(params || {});
                const injector = ReflectiveInjector.resolveAndCreate([
                    { provide: Params, useValue: pageParams },
                    { provide: OnsNavigator, useValue: this }
                ], this._injector);
                const factory = this._resolver.resolveComponentFactory(page);
                const selector = 'ons-navigator';
                const pageComponentRef = factory.create(injector, undefined);
                this._viewContainer.insert(pageComponentRef.hostView);
                const pageElement = pageComponentRef.location.nativeElement;
                componentRefMap.set(pageElement, pageComponentRef);
                this.element.appendChild(pageElement); // dirty fix to insert in correct position
                done(pageElement);
            });
        }, (element) => {
            const componentRef = componentRefMap.get(element);
            if (componentRef) {
                componentRef.destroy();
                componentRefMap.delete(element);
            }
        });
    }
    ngOnDestroy() {
        this.element.pageLoader = this._lastPageLoader;
    }
}
OnsNavigator.decorators = [
    { type: Directive, args: [{
                selector: 'ons-navigator'
            },] },
];
/** @nocollapse */
OnsNavigator.ctorParameters = () => [
    { type: ElementRef, },
    { type: ComponentFactoryResolver, },
    { type: ViewContainerRef, },
    { type: Injector, },
    { type: NgZone, },
];
OnsNavigator.propDecorators = {
    'pageComponentType': [{ type: Input, args: ['page',] },],
};
//# sourceMappingURL=ons-navigator.js.map