import { Injector, Directive, ElementRef, Input, ReflectiveInjector, ViewContainerRef, ComponentFactoryResolver, NgZone } from '@angular/core';
import * as ons from 'onsenui';
import { Params } from '../ons/params';
/**
 * @element ons-splitter-side
 * @directive OnsSplitterSide
 * @selector ons-splitter-side
 * @description
 *   [ja]`<ons-splitter-side>`要素のAngular 2ディレクティブです。[/ja]
 *   [en]Angular 2 directive for `<ons-splitter-side>` component.[/en]
 * @example
 *   @Component({
 *     selector: 'ons-page',
 *     template: `
 *       <ons-toolbar>
 *         <div class="center">Left Page</div>
 *       </ons-toolbar>
 *       <div class="background"></div>
 *       <div class="content">
 *         Left
 *       </div>
 *     `
 *   })
 *   class SidePageComponent { }
 *
 *   @Component({
 *     selector: 'app',
 *     template: `
 *     <ons-splitter>
 *       <ons-splitter-side [page]="sidePage" side="left" width="200px">
 *       </ons-splitter-side>
 *       <ons-splitter-content>...</ons-splitter-content>
 *     </ons-splitter>
 *     `
 *   })
 *   export class AppComponent {
 *     sidePage = SidePageComponent;
 *   }
 */
export class OnsSplitterSide {
    constructor(_elementRef, _viewContainer, _resolver, _injector, _zone) {
        this._elementRef = _elementRef;
        this._viewContainer = _viewContainer;
        this._resolver = _resolver;
        this._injector = _injector;
        this._zone = _zone;
        this.element.pageLoader = this._createPageLoader();
    }
    /**
     * @input page
     * @type {Type<any>}
     * @desc
     *   [en]Page content.[/en]
     *   [ja]表示するページコンポーネントのクラスを指定します。[/en]
     */
    set page(page) {
        this.element.page = page;
    }
    get element() {
        return this._elementRef.nativeElement;
    }
    get nativeElement() {
        return this._elementRef.nativeElement;
    }
    _createPageLoader() {
        const componentRefMap = new WeakMap();
        return new ons.PageLoader(({ page, parent, params }, done) => {
            this._zone.run(() => {
                const injector = ReflectiveInjector.resolveAndCreate([
                    { provide: Params, useValue: new Params(params || {}) },
                    { provide: OnsSplitterSide, useValue: this }
                ], this._injector);
                const factory = this._resolver.resolveComponentFactory(page);
                const pageComponentRef = this._viewContainer.createComponent(factory, 0, injector);
                const pageElement = pageComponentRef.location.nativeElement;
                componentRefMap.set(pageElement, pageComponentRef);
                this.element.appendChild(pageElement); // dirty fix to insert in correct position
                done(pageElement);
            });
        }, (element) => {
            const componentRef = componentRefMap.get(element);
            if (componentRef) {
                componentRef.destroy();
                componentRefMap.delete(element);
            }
        });
    }
}
OnsSplitterSide.decorators = [
    { type: Directive, args: [{
                selector: 'ons-splitter-side'
            },] },
];
/** @nocollapse */
OnsSplitterSide.ctorParameters = () => [
    { type: ElementRef, },
    { type: ViewContainerRef, },
    { type: ComponentFactoryResolver, },
    { type: Injector, },
    { type: NgZone, },
];
OnsSplitterSide.propDecorators = {
    'page': [{ type: Input, args: ['page',] },],
};
/**
 * @element ons-splitter-content
 * @directive OnsSplitterContent
 * @selector ons-splitter-content
 * @description
 *    [ja]`<ons-splitter-content>`要素のためのAngular2 ディレクティブです。[/ja]
 *    [en]Angular 2 directive for `<ons-splitter-content>` component.[/en]
 */
export class OnsSplitterContent {
    constructor(_elementRef, _viewContainer, _resolver, _injector) {
        this._elementRef = _elementRef;
        this._viewContainer = _viewContainer;
        this._resolver = _resolver;
        this._injector = _injector;
        this.element.pageLoader = this._createPageLoader();
    }
    /**
     * @input page
     * @type {Type<any>}
     * @desc
     *   [en]Specify the page component.[/en]
     *   [ja]表示するページコンポーネントのクラスを指定します。[/en]
     */
    set page(page) {
        this.element.page = page;
    }
    get element() {
        return this._elementRef.nativeElement;
    }
    get nativeElement() {
        return this._elementRef.nativeElement;
    }
    _createPageLoader() {
        const componentRefMap = new WeakMap();
        return new ons.PageLoader(({ page, parent, params }, done) => {
            const injector = ReflectiveInjector.resolveAndCreate([
                { provide: Params, useValue: new Params(params || {}) },
                { provide: OnsSplitterContent, useValue: this }
            ], this._injector);
            const factory = this._resolver.resolveComponentFactory(page);
            const pageComponentRef = this._viewContainer.createComponent(factory, 0, injector);
            const pageElement = pageComponentRef.location.nativeElement;
            componentRefMap.set(pageElement, pageComponentRef);
            this.element.appendChild(pageElement); // dirty fix to insert in correct position
            done(pageElement);
        }, (element) => {
            const componentRef = componentRefMap.get(element);
            if (componentRef) {
                componentRef.destroy();
                componentRefMap.delete(element);
            }
        });
    }
}
OnsSplitterContent.decorators = [
    { type: Directive, args: [{
                selector: 'ons-splitter-content'
            },] },
];
/** @nocollapse */
OnsSplitterContent.ctorParameters = () => [
    { type: ElementRef, },
    { type: ViewContainerRef, },
    { type: ComponentFactoryResolver, },
    { type: Injector, },
];
OnsSplitterContent.propDecorators = {
    'page': [{ type: Input, args: ['page',] },],
};
//# sourceMappingURL=ons-splitter.js.map