import { ViewContainerRef, ComponentFactoryResolver, Directive, ElementRef, Input, NgZone } from '@angular/core';
import * as ons from 'onsenui';
/**
 * @element ons-tab
 * @directive OnsTab
 * @selector ons-tab
 * @description
 *   [en]Angular 2 directive for `<ons-tab>` component.[/en]
 *   [ja]`<ons-tab>`要素のためのディレクティブです。[/ja]
 * @example
 *   @Component({
 *     selector: 'ons-page',
 *     template: `
 *       <ons-toolbar>
 *         <div class="center">Page</div>
 *       </ons-toolbar>
 *       <div class="content">...</div>
 *     `
 *   })
 *   class PageComponent {
 *   }
 *
 *   @Component({
 *     selector: 'app',
 *     template: `
 *     <ons-tabbar>
 *       <div class="tabbar__content"></div>
 *       <div class="tabbar">
 *         <ons-tab label="Page1" icon="ion-home" [page]="page" active></ons-tab>
 *         <ons-tab label="Page2" icon="ion-help" [page]="page"></ons-tab>
 *         <ons-tab label="Page3" icon="ion-stop" [page]="page"></ons-tab>
 *       </div>
 *     </ons-tabbar>
 *     `
 *   })
 *   export class AppComponent {
 *     page = PageComponent
 *   }
 */
export class OnsTab {
    constructor(_elementRef, _viewContainer, _resolver, _zone) {
        this._elementRef = _elementRef;
        this._viewContainer = _viewContainer;
        this._resolver = _resolver;
        this._zone = _zone;
        this._pageComponent = null;
        // set up ons-tab's page loader
        this._elementRef.nativeElement.pageLoader = this._createPageLoader();
    }
    /**
     * @input page
     * @type {Type<any>}
     * @desc
     *   [en]Specify the page component that is displayed when the tab is active.[/en]
     *   [ja]読み込むページコンポーネントを指定します。[/ja]
     */
    set page(pageComponentType) {
        this._elementRef.nativeElement.page = pageComponentType;
    }
    _createPageLoader() {
        const PageLoader = ons.PageLoader;
        return new PageLoader(({ page, parent }, done) => {
            this._zone.run(() => {
                const factory = this._resolver.resolveComponentFactory(page);
                const pageComponentRef = this._viewContainer.createComponent(factory, 0);
                if (this._pageComponent) {
                    this._pageComponent.destroy();
                }
                this._pageComponent = pageComponentRef;
                const pageElement = pageComponentRef.location.nativeElement;
                parent.appendChild(pageElement); // dirty fix to insert in correct position
                done(pageElement);
            });
        }, () => {
            if (this.hasOwnProperty('_pageComponent')) {
                if (this._pageComponent) {
                    this._pageComponent.destroy();
                    this._pageComponent = null;
                }
            }
        });
    }
    ngOnDestroy() {
        if (this._pageComponent) {
            this._pageComponent.destroy();
            this._pageComponent = null;
        }
    }
}
OnsTab.decorators = [
    { type: Directive, args: [{
                selector: 'ons-tab'
            },] },
];
/** @nocollapse */
OnsTab.ctorParameters = () => [
    { type: ElementRef, },
    { type: ViewContainerRef, },
    { type: ComponentFactoryResolver, },
    { type: NgZone, },
];
OnsTab.propDecorators = {
    'page': [{ type: Input, args: ['page',] },],
};
//# sourceMappingURL=ons-tabbar.js.map